@@ -61,7 +61,7 @@ def set_random_seed(seed):
 class DensityFlow(nn.Module):
     def __init__(self,
                  input_size: int,
-                 codebook_size: int = 30,
+                 codebook_size: int = 15,
                  cell_factor_size: int = 0,
                  specific_mode: Literal['none','codebook','cell'] = 'codebook',
                  z_dim: int = 50,
@@ -92,8 +92,6 @@ class DensityFlow(nn.Module):
         self.allow_broadcast = config_enum == 'parallel'
         self.use_cuda = use_cuda
         self.loss_func = loss_func
-        if (specific_mode == 'codebook') and (loss_func=='negbinomial'):
-            self.loss_func == 'poisson'
         self.options = None
         self.code_size=codebook_size
         self.latent_dist = z_dist
@@ -402,10 +400,20 @@ class DensityFlow(nn.Module):
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                if self.use_zeroinflate:
-                    pyro.sample("x", MyZINB(mu=mu, theta=dispersion, zi_logits=gate_logits).to_event(1), obs=xs)
+                if self.specific_mode=='codebook':
+                    logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
+                    if self.use_zeroinflate:
+                        pyro.sample('x', 
+                                    dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),
+                                                                  gate_logits=gate_logits).to_event(1), 
+                                    obs=xs)
+                    else:
+                        pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
                 else:
-                    pyro.sample("x", MyNB(mu=mu, theta=dispersion).to_event(1), obs=xs)
+                    if self.use_zeroinflate:
+                        pyro.sample("x", MyZINB(mu=mu, theta=dispersion, zi_logits=gate_logits).to_event(1), obs=xs)
+                    else:
+                        pyro.sample("x", MyNB(mu=mu, theta=dispersion).to_event(1), obs=xs)
             elif self.loss_func == 'poisson':
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.Poisson(rate=rate),gate_logits=gate_logits).to_event(1), obs=xs.round())
@@ -455,14 +463,17 @@ class DensityFlow(nn.Module):
         cb = self._get_codebook()
         cb = tensor_to_numpy(cb)
         return cb
-    
-    def _codebook_map(self, xs):
-        ns = self._soft_assignments(xs)
+
+    def _codebook_map(self, xs, soft_assign):
+        if soft_assign:
+            ns = self._soft_assignments(xs)
+        else:
+            ns = self._hard_assignments(xs)
         cb = self._get_codebook()
         zs = torch.matmul(ns, cb)
         return zs
     
-    def codebook_map(self, xs, batch_size:int=1024, show_progress=True):
+    def codebook_map(self, xs, soft_assign: bool=True, batch_size:int=1024, show_progress=True):
         xs = self.preprocess(xs)
         xs = convert_to_tensor(xs, device=self.get_device())
         dataset = CustomDataset(xs)
@@ -471,7 +482,7 @@ class DensityFlow(nn.Module):
         Z = []
         with tqdm(total=len(dataloader), disable=not show_progress, desc='', unit='batch') as pbar:
             for X_batch, _ in dataloader:
-                zns = self._codebook_map(X_batch)
+                zns = self._codebook_map(X_batch, soft_assign)
                 Z.append(tensor_to_numpy(zns))
                 pbar.update(1)
 
@@ -657,6 +668,7 @@ class DensityFlow(nn.Module):
                              xs, 
                              perturb_idx,
                              perturb_us,
+                             soft_assign: bool = True,
                              batch_size: int = 1024,
                              show_progress=True):
         """
@@ -665,9 +677,9 @@ class DensityFlow(nn.Module):
         """
         #xs = self.preprocess(xs)
         if (self.specific_mode == 'codebook'):
-            zs = self.codebook_map(xs)
+            zs = self.codebook_map(xs, soft_assign=soft_assign, show_progress=show_progress)
         else:
-            zs = self.get_basal_embedding(xs)
+            zs = self.get_basal_embedding(xs, show_progress=show_progress)
         zs = convert_to_tensor(zs, device=self.get_device())
         ps = convert_to_tensor(perturb_us, device=self.get_device())
         dataset = CustomDataset2(zs,ps)
@@ -747,6 +759,8 @@ class DensityFlow(nn.Module):
             xs = ad.X.copy()
         elif self.loss_func in ['poisson','multinomial']:
             xs = np.round(xs)
+        elif self.specific_mode=='codebook': # negbinomial
+            xs = np.round(xs)
             
         if sparse.issparse(xs):
             xs = xs.toarray()
