@@ -7,6 +7,7 @@ from matplotlib.colors import ListedColormap
 
 def plot_quiver(z_points, delta_z, method='pca', figsize=(6,4), dpi=200, 
                 subsample=None, color_by=None, colormap='viridis', 
+                point_size = 3, point_alpha=0.5,
                 arrow_scale=1.0, arrow_width=0.002, alpha=0.8):
     """
     优化后的quiver可视化函数
@@ -97,8 +98,11 @@ def plot_quiver(z_points, delta_z, method='pca', figsize=(6,4), dpi=200,
             cbar_label = 'Numeric Label'
         else:  # 类别型标签
             unique_labels = np.unique(original_color_by)
-            color_map = {label: cmap(i/len(unique_labels)) 
-                        for i, label in enumerate(unique_labels)}
+            if type(cmap)==dict:
+                color_map = colormap
+            else:
+                color_map = {label: cmap(i/len(unique_labels)) 
+                            for i, label in enumerate(unique_labels)}
             colors_all = [color_map[label] for label in original_color_by]
             if color_by_quiver is not None:
                 colors_quiver = [color_map[label] for label in color_by_quiver]
@@ -113,8 +117,27 @@ def plot_quiver(z_points, delta_z, method='pca', figsize=(6,4), dpi=200,
     plt.figure(figsize=figsize, dpi=dpi)
     
     # 1. 首先绘制所有数据点的散点图
-    plt.scatter(z_2d_all[:, 0], z_2d_all[:, 1], 
-               c=colors_all, alpha=0.3, s=5, label='All data points')
+    if (original_color_by is not None) and (unique_labels is not None):
+        for label in unique_labels:
+            ind = color_by == label
+            colors_label = [colors_all[ii] for ii in np.arange(z_2d_all.shape[0]) if ind[ii]]
+            plt.scatter(z_2d_all[ind, 0], z_2d_all[ind, 1], 
+                   c=colors_label, alpha=point_alpha, s=point_size, label=label)
+            
+        plt.legend(
+        ncol = max(1, len(unique_labels) // 8),
+        bbox_to_anchor=(1.00, 0.5),  # 图外右侧中间
+        loc='center left',            # 锚点在左侧中间
+        borderaxespad=0.0,            # 与图的间距
+        frameon=False,                 # 显示边框
+        fancybox=False,                # 圆角
+        shadow=False,                  # 阴影
+        #handletextpad=0.5,           # 点与文本间距
+        markerscale=min(2*point_size,4)           # 关键：调整图例中点的大小（2倍）
+        )
+    else:
+        plt.scatter(z_2d_all[:, 0], z_2d_all[:, 1], 
+                   c=colors_all, alpha=point_alpha, s=point_size)
     
     # 2. 绘制quiver（仅采样数据）
     if color_by_quiver is not None and isinstance(color_by_quiver[0], str):
@@ -123,15 +146,12 @@ def plot_quiver(z_points, delta_z, method='pca', figsize=(6,4), dpi=200,
             plt.quiver(z_2d_quiver[mask, 0], z_2d_quiver[mask, 1],
                       delta_z_2d[mask, 0], delta_z_2d[mask, 1],
                       angles='xy', scale_units='xy', scale=1.0/arrow_scale,
-                      color=colors_quiver[mask], width=arrow_width, alpha=alpha,
-                      label=f'{label} (movement)')
-        plt.legend()
+                      color=colors_quiver[mask], width=arrow_width, alpha=alpha)
     else:
         q = plt.quiver(z_2d_quiver[:, 0], z_2d_quiver[:, 1],
                       delta_z_2d[:, 0], delta_z_2d[:, 1],
                       angles='xy', scale_units='xy', scale=1.0/arrow_scale,
-                      color=colors_quiver, width=arrow_width, alpha=alpha,
-                      label='Movement vectors')
+                      color=colors_quiver, width=arrow_width, alpha=alpha)
         
         # 添加颜色条（数值型标签）
         if original_color_by is not None and original_color_by.dtype.kind in ['i', 'f']:
@@ -144,7 +164,6 @@ def plot_quiver(z_points, delta_z, method='pca', figsize=(6,4), dpi=200,
     plt.ylabel(dim_names[1])
     plt.title(f"Latent Space Movement ({method} projection)\n{len(z_points_quiver)}/{len(original_z_points)} points with vectors")
     plt.grid(alpha=0.2)
-    plt.legend()
     plt.tight_layout()
     plt.show()
     
